/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <biorob-cli/Interface.h>

using namespace cli;

class BindCallback : public KeyCallback {
public :
	virtual ~BindCallback(){}
	virtual int Call(int key){
		std::cerr << "<<< You pressed me with " << unctrl((chtype)key) << std::endl;
		return 0;
	}
};

class PrintCommand : public Command {
public :
	PrintCommand() : Command("print",0,true){}
	virtual ~PrintCommand(){}

	bool Call(const ListOfArguments & args){
		std::cout <<"<<< [print] : ";
		if(args.empty()){
			std::cout << "No arguments passed !" << std::endl;
			return true;
		}
		for(ListOfArguments::const_iterator a = args.begin();
		    a != args.end();
		    ++a){
			if(a != args.begin()){
				std::cout << " , ";
			}
			std::cout << "(" << (*a)->Name() << " : " << (*a)->toString()
			          << " )";
		}
		std::cout << std::endl;

		return true;
	}
};


class EchoCommand : public Command {
public :
	EchoCommand() : Command("echo",1,false){}
	virtual ~EchoCommand() {}

	bool Call(const ListOfArguments & args){
		std::cout << "<<< [echo] : " << args[0]->Name() << " : "
		          << args[0]->toString() << std::endl;
		return true;
	}
};

int main(){
	Interface i(0,0,true);
	try{
		i.AddVariable(VariableBase::Ptr(new Variable<double>("foo",0.0)));
		i.AddVariable(VariableBase::Ptr(new Variable<double>("bar",1.0)));

		i.AddCommand(Command::Ptr(new PrintCommand()));
		i.AddCommand(Command::Ptr(new EchoCommand()));
		i.AddBinding('+',KeyCallback::Ptr(new BindCallback()));
		i.AddBinding('-',KeyCallback::Ptr(new BindCallback()));

		while(true){
			i.Process();
			if(i.UserAskedToQuit()){
				break;
			}
		}
	} catch (...) {
		Interface::SignalDeinitFunction(0);
		throw;
	}

	return 0;

}
