/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file DisplayWidget.cpp
 *
 *  \date Oct 24, 2012
 *  \author Alexandre Tuleu
 */

#include "DisplayWidget.h"
#include "Interface.h"
namespace cli {

	DisplayWidget::DisplayWidget(const Widget::Def & def)
		: Widget(def)
		, d_stream(new std::ostream(new StreamBuf(Window())))
		, d_errStream(new std::ostream(new StreamBuf(Window(),
		                                             Interface::HasColor() ? Interface::RedOnBlack
		                                                                   : -1))){
		scrollok(Window(),true);
	}

	DisplayWidget::~DisplayWidget(){
	}

	DisplayWidget::StreamBuf::StreamBuf(WINDOW* w, int colorPair, std::size_t buffSize)
		: d_w(w), d_buffer(buffSize + 1), d_colorPair(colorPair){
		char * base = &d_buffer.front();
		setp(base,base + d_buffer.size() - 1);
	}

	DisplayWidget::StreamBuf::~StreamBuf(){}

	int DisplayWidget::StreamBuf::sync(){
		return flush() ? 0 : -1;
	}

	int DisplayWidget::StreamBuf::overflow(int c){
		if(c == traits_type::eof()){
			return c;
		}

		*pptr() = c;
		pbump(1);
		if(flush()){
			return c;
		}
		return traits_type::eof();
	}

	bool DisplayWidget::StreamBuf::flush(){
		std::ptrdiff_t n = pptr() - pbase();
		pbump(-n);
		if(d_colorPair >= 1){
			wattrset(d_w,(attr_t)COLOR_PAIR(d_colorPair));
		}
		bool res = waddnstr(d_w,pbase(),n) == OK;
		if(d_colorPair >= 1){
			wattrset(d_w,COLOR_PAIR(Interface::WhiteOnBlack));
		}
		wrefresh(d_w);
		return res;
	}

	std::ostream & DisplayWidget::Stream(){
		return * d_stream;
	}

	std::ostream & DisplayWidget::ErrStream(){
		return * d_errStream;
	}

} /* namespace cli */
