/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Copyright (c) 2012  Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Interface.h
 *
 *  \date Oct 24, 2012
 *  \author Alexandre Tuleu
 */

#ifndef BIOROB_CLI_INTERFACE_H_
#define BIOROB_CLI_INTERFACE_H_

#include "Prompt.h"
#include "DisplayWidget.h"
#include "Variable.h"
#include "Command.h"

namespace cli {
/**
 *
 */
class Interface {
public:
	typedef std::map<std::string,VariableBase::Ptr> VariableByName;

	Interface(unsigned int rows, unsigned int cols, bool handleSignals = true);
	virtual ~Interface();


	template <class WidgetClass>
	std::tr1::shared_ptr<WidgetClass> AddWidget(const std::string & title, unsigned int row, unsigned int col, unsigned int spanRow = 1,unsigned int spanCol = 1);

	void AddVariable(VariableBase::Ptr var);
	void AddCommand(Command::Ptr command);
	void RemoveCommand(const std::string & name);
	void RemoveVariable(VariableBase::Ptr var);

	void AddBinding(int c,KeyCallback::Ptr callback);
	void Unbind(int c);

	bool Process();

	bool UserAskedToQuit();

	const std::string & GetPrompt() const;
	void SetPrompt(const std::string & p);

	const VariableByName & Variables() const;


	std::ostream & IOut();
	std::ostream & IErr();

	static void SignalDeinitFunction(int sig);

	enum Colors{
		WhiteOnBlack = 1,
		GreenOnBlack = 2,
		RedOnBlack   = 3
	};

	static bool HasColor();

	static void InitializeNCurses();
	static void OutOfCurses();
	static void InCurses();
private :
	typedef std::map<std::string,Command::Ptr> CommandByName;
	typedef std::map<int,KeyCallback::Ptr> CallBackByKeys;

	static void DeInitializeNCurses();
	void InitializeSizes();

	void SplitInToken(const std::string & value,
	                  std::vector<std::string> & tokens) const;

	void CallCommand(CommandByName::iterator & cmd,
	                 std::vector<std::string> & tokens);

	bool ExtractArgs(std::vector<std::string> & tokens,
	                 Command::ListOfArguments & args);

	void ProcessVariable(VariableByName::iterator & var,
	                     std::vector<std::string> & tokens);

	static const std::string SpecialChars;
	static const std::string SpaceChars;

	static bool s_alreadyExists;
	static bool s_ncursesInitialized;

	static const unsigned int MinPromptHeight;
	static const unsigned int MaxPromptHeight;
	static const unsigned int MinWidgetSize;
	unsigned int d_widgetHeight,d_widgetWidth, d_promptHeight, d_displayHeight,
	d_rows,d_cols;
	static bool s_hasColor;

	std::tr1::shared_ptr<Prompt> d_prompt;
	std::tr1::shared_ptr<DisplayWidget> d_generalDisplay;

	static std::streambuf * s_coutBuf, * s_cerrBuf, * s_clogBuf;
	VariableByName d_variables;
	CommandByName  d_commands;
	bool d_userAskToQuit;
	std::string d_promptString;

};

template <typename WidgetClass>
inline std::tr1::shared_ptr<WidgetClass> Interface::AddWidget(const std::string & title, unsigned int row, unsigned int col, unsigned int spanRow, unsigned int spanCol){
	if(spanCol == 0 || spanRow == 0 || row >= d_rows || col >= d_cols){
		throw std::runtime_error("Bad position and size definition");
	}

	Widget::Def def;
	def.y = (int)row * (int)d_widgetHeight;
	def.x = (int)col * (int)d_widgetWidth;
	if(col == d_cols - spanCol){
		def.width = 0;
	} else {
		def.width = (int)(spanCol * d_widgetWidth);
	}

	def.height = (int)(spanRow * d_widgetHeight);
	def.title = title;
	return std::tr1::shared_ptr<WidgetClass>(new WidgetClass(def));
}

}
#endif // BIOROB_CLI_INTERFACE_H_
