/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * Copyright (c) 2012  Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   PROJECT.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Widget.cpp
 *
 *  \date Oct 24, 2012
 *  \author Alexandre Tuleu
 */

#include "Widget.h"
#include <stdexcept>
#include "Interface.h"

namespace cli {
	Widget::Def::Def() : x(0), y(0),height(0),width(0),title() {}

	Widget::Widget(const Def & def)
		: d_window(newwin(def.height - 1,def.width,def.y + 1,def.x))
		, d_titleWindow(newwin(1,def.width,def.y,def.x))
		, d_width(0)
		, d_height(){

		if(d_window == NULL){
			std::ostringstream os;
			os << "Could not open main window. Def was " << def;
			throw std::runtime_error(os.str());
		}
		if(d_titleWindow == NULL){
			std::ostringstream os;
			os << "Could not open title window. Def was " << def;
			throw std::runtime_error(os.str());
		}
		d_width = (std::size_t)getmaxx(d_window);
		d_height = (std::size_t)getmaxy(d_window);

		wattrset(d_window,COLOR_PAIR(Interface::WhiteOnBlack));
		wattrset(d_titleWindow,COLOR_PAIR(Interface::WhiteOnBlack));
		SetTitle(def.title);
	}

	Widget::~Widget(){
		delwin(d_window);
		delwin(d_titleWindow);
	}

	std::size_t Widget::Width(){
		return d_width;
	}

	std::size_t Widget::Height(){
		return d_height;
	}

	void Widget::SetTitle(const std::string & title){
		std::string actualTitle = "-*- " + title + " -*-";
		if(actualTitle.size() > d_width){
			actualTitle = "-*- " + title.substr(0,actualTitle.size() - d_width - 3) + "... -*-";
		}
		int xPos = (int)(d_width - actualTitle.size()) / 2;
		if(Interface::HasColor()){
			wattrset(d_titleWindow,COLOR_PAIR(Interface::GreenOnBlack));
		}
		mvwaddstr(d_titleWindow,0,xPos,actualTitle.c_str());
		wrefresh(d_titleWindow);
	}

}/* namespace cli */

std::ostream & operator << (std::ostream & out, const cli::Widget::Def & def){
	return out << "{title : '" << def.title << "' size : " << def.width << "x"
	           << def.height << " at ( " << def.x << " , " << def.y << " )}";
}
