/*
 * Copyright (c) 2012 - 2013 Alexandre Tuleu
 *
 * This file is part of biorob-cpp.
 *
 *   biorob-cpp is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU Lesser General Public License as
 *   published by the Free Software Foundation, either version 3 of the
 *   License, or (at your option) any later version.
 *
 *   biorob-cpp is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of both the GNU General Public
 *   License AND GNU Lesser General Public License along with
 *   biorob-cpp.  If not, see <http://www.gnu.org/licenses/>.
 */


/**
 * \file Logger.cpp
 *
 *  \date Jul 26, 2012
 *  \author Alexandre Tuleu
 */

#include "Logger.h"
#include <stdexcept>

#include <fstream>

Logger::Logger(const std::string & name)
	: d_stream(&std::cerr)
	, d_deleteStream(false)
	, d_name(name){
}

Logger::~Logger(){
	Clean();
}

std::ostream& Logger::Stream(){
	return *d_stream;
}

void Logger::SetNull(){
	Clean();
	d_stream = & s_nullStream;
	d_deleteStream = false;
}

void Logger::SetFile(const std::string& filename){
	Clean();
	std::ofstream * file = new std::ofstream();
	std::ios::iostate oldstate = file->exceptions();
	file->exceptions(oldstate | std::ofstream::failbit | std::ofstream::badbit);
	file->open(filename.c_str(), std::ios_base::app);
	d_stream = file;
	file->exceptions(oldstate);
	d_deleteStream = true;
}

void Logger::SetCout(){
	Clean();
	d_stream = & std::cout;
	d_deleteStream = false;
}

void Logger::SetCerr(){
	Clean();
	d_stream = & std::cerr;
	d_deleteStream = false;
}

void Logger::SetClog(){
	Clean();
	d_stream = & std::clog;
	d_deleteStream = false;
}

void Logger::Clean(){
	if(d_stream == NULL || !d_deleteStream){
		return;
	}
	delete d_stream;
	d_deleteStream = false;
	d_stream = NULL;
}

std::ostream Logger::s_nullStream(new Logger::NullStreamBuff());


bool Logger::AppendName() const{
	return !d_deleteStream;
}

const std::string & Logger::Name() const{
	return d_name;
}
